package be.iminds.ilabt.jfed.experimenter_gui.slice.espec.view;

import be.iminds.ilabt.jfed.experimenter_gui.slice.espec.model.ESpecLogItem;
import be.iminds.ilabt.jfed.experimenter_gui.slice.espec.model.ESpecLogModel;
import be.iminds.ilabt.jfed.experimenter_gui.slice.espec.model.ESpecStateItem;
import be.iminds.ilabt.jfed.experimenter_gui.ui.status.TaskStatusIndicator;
import javafx.beans.binding.ObjectBinding;
import javafx.beans.property.ObjectProperty;
import javafx.beans.property.StringProperty;
import javafx.collections.ListChangeListener;
import javafx.scene.Cursor;
import javafx.scene.control.Label;
import javafx.scene.control.ListCell;
import javafx.scene.control.ListView;
import javafx.scene.control.Tooltip;
import javafx.scene.layout.HBox;

import javax.annotation.Nonnull;


public class ESpecLogList extends ListView<ESpecLogItem> {


    public ESpecLogList(ESpecLogModel eSpecLogModel) {
        setPrefHeight(100);
        setCellFactory(progressItemListView -> new ESpecViewItem());

        setItems(eSpecLogModel.getESpecLogItems());

        //always show the last progressitem that was added
        ListChangeListener<ESpecLogItem> showLastItem = c -> {
            while (c.next()) {
                if (c.wasAdded()) {
                    scrollTo(getItems().size() - 1);
                }
            }
        };
        getItems().addListener(showLastItem);

        eSpecLogModel.selectedESpecStateItemProperty().addListener((o, oldVal, newVal) -> {
            if (oldVal != newVal) {
                getItems().removeListener(showLastItem);
                if (newVal != null) {
                    setItems(newVal.getLogItems());
                } else {
                    setItems(eSpecLogModel.getESpecLogItems());
                }
                getItems().addListener(showLastItem);
                scrollTo(getItems().size() - 1);
            }
        });
    }


    private static class ESpecViewItem extends ListCell<ESpecLogItem> {

        private final HBox hbox;

        private final TaskStatusIndicator taskStatusIndicator;
        private final Label textLabel;
        private final Label statusLabel;

        public ESpecViewItem() {
            hbox = new HBox();
            hbox.setSpacing(5);


            // add taskStatusIndicator as graphic
            taskStatusIndicator = new TaskStatusIndicator();
            taskStatusIndicator.setPrefHeight(16);
            taskStatusIndicator.setPrefWidth(16);

            textLabel = new Label();
            statusLabel = new Label();
            statusLabel.setStyle("-fx-text-fill: darkgrey;");

            hbox.getChildren().addAll(taskStatusIndicator, textLabel, statusLabel);

            setGraphic(hbox);
        }

        @Override
        protected void updateItem(final ESpecLogItem progressItem, boolean empty) {
            super.updateItem(progressItem, empty);

            if (!empty) {
                textLabel.tooltipProperty().bind(new TooltipBinding(progressItem.eSpecStateItemProperty()));
                textLabel.textProperty().bind(progressItem.textProperty());
                statusLabel.textProperty().bind(progressItem.statusProperty());
                taskStatusIndicator.statusProperty().bind(progressItem.progressProperty());
                taskStatusIndicator.setVisible(true);

                setOnMouseEntered(mouseEvent -> {
                    if (progressItem.getOnDetailsView() != null) {
                        setOnMouseClicked(progressItem.getOnDetailsView());
                        setCursor(Cursor.HAND);
                    } else {
                        setOnMouseClicked(null);
                        setCursor(null);
                    }
                });

            } else {
                textLabel.textProperty().unbind();
                textLabel.setText("");
                statusLabel.textProperty().unbind();
                statusLabel.setText("");
                taskStatusIndicator.setVisible(false);
                setOnMouseClicked(null);
                setCursor(null);
            }
        }
    }





    private static class TooltipStringBinding extends ObjectBinding<Tooltip> {
        @Nonnull private final StringProperty text;

        public TooltipStringBinding(StringProperty text) {
            bind(text);
            this.text = text;
        }

        @Override
        protected Tooltip computeValue() {
            return new Tooltip(text.get());
        }
    }
    private static class TooltipBinding extends ObjectBinding<Tooltip> {
        @Nonnull private final ObjectProperty<ESpecStateItem> item;

        public TooltipBinding(ObjectProperty<ESpecStateItem> item) {
            bind(item);
            this.item = item;
        }

        @Override
        protected Tooltip computeValue() {
            if (item.get() != null)
                return new Tooltip("Occured during state "+item.get().getHumanName());
            else
                return new Tooltip("Occured during null state ");
        }
    }
}
